components = {
    'paper-input': '@polymer/paper-input/paper-input.js',
    'paper-input-container': '@polymer/paper-input/paper-input-container.js',
    'paper-input-error': '@polymer/paper-input/paper-input-error.js',
    'paper-input-char-counter': '@polymer/paper-input/paper-input-char-counter.js',
    'paper-textarea': '@polymer/paper-input/paper-textarea.js',
}
