<html>
  <head>
  </head>
  <body>
    <paper-input always-float-label label="Floating label"></paper-input>
    <paper-textarea always-float-label label="Floating label"></paper-textarea>
  </body>
</html>
